#ifndef ElementReco_h
#define ElementReco_h

#include "util/include/ActiveObserver.h"
#include "AnalysisFramework/AnalysisSteering.h"
#include <string>
#include <vector>

class Event;
class BraggMean;
class TH1F;

class ElementReco: public AnalysisSteering,
                   public ActiveObserver<Event> {

 public:

  ElementReco( const AnalysisInfo* info );
  virtual ~ElementReco();

  // function to be called at execution start
  virtual void beginJob();
  // function to be called at execution end
  virtual void   endJob();
  // function to be called for each event
  virtual void update( const Event& ev );

 private:

  // dummy copy constructor and assignment to prevent unadvertent copy
  ElementReco           ( const ElementReco& x );
  ElementReco& operator=( const ElementReco& x );

  TH1F* hTot;

  // Bragg curve information
  struct BraggCurve {
    std::string name;  // element name
    BraggMean*  bMean; // statistic object
    TH1F*       hMean; // graph
  };

  // set of Bragg curves for different total energies
  std::vector<BraggCurve*> bCurve;

  // create objects for a Bragg curve
  void bCreate( const std::string& name, float min, float max );

};

#endif

